#ifndef NO_RCSID
const char* QN_Logger_rcsid =
    "$Header: /u/drspeech/repos/quicknet2/QN_Logger.cc,v 1.3 1996/07/23 22:33:37 davidj Exp $";
#endif

// User interaction routines.

/* Must include the config.h file first */
#include <QN_config.h>
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "QN_types.h"
#include "QN_Logger.h"

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS (0)
#define EXIT_FAILURE (1)
#endif

////////////////////////////////////////////////////////////////
// Friendly interfaces to the va_list based logging functions.

void
QN_Logger::error(const char* objname, const char* format, ...)
{
    va_list va;

    va_start(va, format);
    verror(objname, format, va);
    va_end(va);
}

void
QN_Logger::warning(const char* objname, const char* format, ...)
{
    va_list va;

    va_start(va, format);
    vwarning(objname, format, va);
    va_end(va);
}

void
QN_Logger::log(const char* objname, const char* format, ...)
{
    va_list va;

    va_start(va, format);
    vlog(objname, format, va);
    va_end(va);
}

void
QN_Logger::output(const char* format, ...)
{
    va_list va;

    va_start(va, format);
    vlog(NULL, format, va);
    va_end(va);
}

////////////////////////////////////////////////////////////////

QN_ClassLogger::QN_ClassLogger(int a_debug, const char* a_objname,
			       const char* a_instname)
    : debug(a_debug),
      name(new char[strlen(a_objname)+strlen(a_instname) + 2])
{
    strcpy(name, a_objname);
    strcat(name, ".");
    strcat(name, a_instname);
}

QN_ClassLogger::~QN_ClassLogger()
{
    delete[] name;
}

void
QN_ClassLogger::output(const char* format, ...) const
{
    va_list va;
    
    va_start(va, format);
    QN_logger->vlog(NULL, format, va);
    va_end(va);
}

void
QN_ClassLogger::log(int level, const char* format, ...) const
{
    va_list va;
    
    va_start(va, format);
    if (debug>=level)
    {
	QN_logger->vlog(name, format, va);
    }
    va_end(va);
}

void
QN_ClassLogger::error(const char* format, ...) const
{
    va_list va;
    
    va_start(va, format);
    QN_logger->verror(name, format, va);
    va_end(va);
}

void
QN_ClassLogger::warning(const char* format, ...) const
{
    va_list va;
    
    va_start(va, format);
    QN_logger->vwarning(name, format, va);
    va_end(va);
}

